indexing
   description : "Memory-mapped File"
   project     : "Eiffel-mmap"
   author      : "Pirmin Kalberer <kalberer@spin.ch>"
   copyright   : "Copyright (c) 1999, Pirmin Kalberer"
   date        : "$Date: 1999/12/04 23:12:02 $"
   revision    : "$Revision: 1.1 $"

class EMMAP_FILE
   
creation
   make_open_read, make_open_write
   
feature { NONE } -- Implementation
   
   fd_, map_: INTEGER
	 -- Unix file descriptor

   addr_:  POINTER
   
feature -- Creation
   
   make_open_read (fn: STRING) is
	 -- Open file object with `fn' as file name in read-only mode.
      require
	 string_exists: fn /= Void;
	 string_not_empty: not fn.empty
      do
	 emmap_file_open (fn, 0)
      end

   make_open_read_write (fn: STRING) is
	 -- Open file object with `fn' as file name in read and write mode.
      require
	 string_exists: fn /= Void;
	 string_not_empty: not fn.empty
      do
	 emmap_file_open (fn, 1)
      end

   close is
      do
	 emmap_close (addr_, size, fd_, map_)
      end
         
feature

   last_error: INTEGER
	 -- 0: OK
	 -- 1: Failed to open file
	 -- 2: Failed to get file size
	 -- 3: Failed to map file
	 -- 4: Failed to get address of mapped file

   size: INTEGER
  
feature { NONE } -- Implementation
   
   emmap_file_open(fn: STRING; mode: INTEGER) is
	 -- Open file object with `fn' as file name.
	 -- mode=0 for read mode and mode=1 for read/write mode.
      require
	 string_exists: fn /= Void;
	 string_not_empty: not fn.empty
      do
	 last_error := emmap_open (fn.to_external, mode, $addr_, $size, $fd_, $map_)
      end
         
feature { NONE } -- Externals
   
   emmap_open (pathname : POINTER; open_write_ : INTEGER; addr__, size__, fd__, map__: POINTER ) : INTEGER is
      require
	 pointers_not_void: addr__ /= Void and size__ /= Void fd__ /= Void and map__ /= Void
      external "C_InlineWithoutCurrent"
      end

   emmap_close (addr__: POINTER; size__, fd__, map__: INTEGER ) is
      external "C_InlineWithoutCurrent"
      end

end
