/*
   description : "System calls"
   project     : "Eiffel-mmap"
   author      : "Pirmin Kalberer <kalberer@spin.ch>"
   copyright   : "Copyright (c) 1999, Pirmin Kalberer"
   date        : "$Date: 1999/12/04 23:12:02 $"
   revision    : "$Revision: 1.1 $"
*/

/*
  This file is based on the PJL C++ Library by Paul J. Lucas,
  which is part of SWISH++ <http://www.best.com/~pjl/software/swish/>
*/


#include <sys/types.h>				/* for off_t */

#ifdef	WIN32
#include <windows.h>
#endif

// standard
#include <fcntl.h>				/* for open(2), O_RDONLY, ... */
#ifndef	WIN32
#include <sys/mman.h>				/* for mmap(2) */
#include <unistd.h>				/* for close(2) */
#endif
#include <sys/stat.h>				/* for stat(2) */

#ifdef	WIN32
	typedef DWORD size_type;
	typedef HANDLE	fd_type;
#else
	typedef off_t size_type;
	typedef int		fd_type;
#endif

//*****************************************************************************
//
// SYNOPSIS
//
	int emmap_open( char const *path, int open_write, void **addr_, size_type *size_, fd_type *fd_, fd_type *map_ )
//
// DESCRIPTION
//
//	Open and mmap a file.
//
// RETURN VALUE
//
//	Returns 0 if the file was mmapped successfully.
//
// SEE ALSO
//
//	mmap(2), open(2), stat(2)
//
//*****************************************************************************
{
#ifndef	WIN32
	struct stat stat_buf;
	int prot = PROT_NONE;
#endif

#ifdef	WIN32

	*fd_ = CreateFile( path,
		open_write ? GENERIC_WRITE : GENERIC_READ,
		0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL
	);
	if ( *fd_ == INVALID_HANDLE_VALUE ) {
		*fd_ = 0;
		return 1;
	}

	if ( ( *size_ = GetFileSize( *fd_, 0 ) ) == 0xFFFFFFFF ) {
		return 2;
	}

	*map_ = CreateFileMapping( *fd_, NULL,
		open_write ? PAGE_READWRITE : PAGE_READONLY,
		0, 0, NULL
	);
	if ( !*map_ ) {
		return 3;
	}

	*addr_ = MapViewOfFile( *map_,
		open_write ? FILE_MAP_WRITE : FILE_MAP_READ,
		0, 0, 0
	);
	if ( !*addr_ ) {
		return 4;
	}
#else
	if ( ( *fd_ = open( path, open_write ? O_RDWR : O_RDONLY ) ) == -1 ) {
		*fd_ = 0;
		return 1;
	}

	if ( stat( path, &stat_buf ) == -1 ) {
		return 2;
	}

	prot |= PROT_READ;
	if ( open_write ) prot |= PROT_WRITE;

	if ( stat_buf.st_size ) {
		*addr_ = mmap( 0, stat_buf.st_size, prot, MAP_SHARED, *fd_, 0 );
		if ( *addr_ == ( caddr_t )( -1 ) ) {
			*addr_ = 0;
			return 3;
		}
	} /* else ???? */

	*size_ = stat_buf.st_size;

#endif	/* WIN32 */

	return 0;
}

//*****************************************************************************
//
// SYNOPSIS
//
	void emmap_close(void *addr_, size_type size_, fd_type fd_, fd_type map_)
//
// DESCRIPTION
//
//	Munmaps and closes a file previously opened and mmapped by
//	emmap_open().
//
// SEE ALSO
//
//	close(2), munmap(2)
//
//*****************************************************************************
{
#ifdef	WIN32
	if ( addr_ )
		UnmapViewOfFile( addr_ );
	if ( map_ )
		CloseHandle( map_ );
	if ( fd_ )
		CloseHandle( fd_ );
#else
	if ( addr_ )
	  	munmap( ( char* )( addr_ ), size_ );
	if ( fd_ )
		close( fd_ );
#endif
}
